package numericalmethod.cauchyproblem.utils;

import static java.lang.Math.*;

public class Initialization {
    private int n;

    private double[] t;
    private double[] x1, x2;

    /**
     * Инициализация начальных данных
     * @param t0 начальный момент времени
     * @param h шаг времени
     * @param stepCount количество шагов времени
     * @param n размерность квадратной матрицы
     * @param indexNonZeroCoordinateFirstInitialVector ненулевая координата первого начального вектора
     * @param indexNonZeroCoordinateSecondInitialVector ненулевая координата второго начального вектора
     */
    public Initialization(double t0, double h, int stepCount,
            int n, int indexNonZeroCoordinateFirstInitialVector, int indexNonZeroCoordinateSecondInitialVector) {
        // Создание системы равноотстоящих узлов
        this.n = n;
        t = new double[stepCount + 1];
        for (int i = 0; i <= stepCount; i++) {
            t[i] = t0 + h * i;
        }

        // Инициализация первого начального вектора
        x1 = new double[n + 1];
        x1[indexNonZeroCoordinateFirstInitialVector] = 1d;

        // Инициализация второго начального вектора
        x2 = new double[n + 1];
        x2[indexNonZeroCoordinateSecondInitialVector] = 1d;
    }

    public double[] getT() {
        return t;
    }

    public double[] getX1() {
        return x1;
    }

    public double[] getX2() {
        return x2;
    }

    // При необходимости изменить определение функции
    public double[] getF(double t, double[] x) {
        double[] f = new double[n + 1];

        double arg = 2d * Math.PI * t;
        double lambda = 2d + sin(arg);
        double mu = 2d + cos(arg);

        f[0] = -lambda * x[0] + mu * x[1];
        f[n] = lambda * x[0] - n * mu * x[n];
        for (int i = 1; i < n; i++) {
            f[i] = -i * mu * x[i] + (i + 1) * mu * x[i + 1];
        }

        return f;
    }
}
